#! /bin/bash
#
# customize_web.sh
# Customizes the web pages and updates xml/config.xml for the following:
# 1) Model (ie 'SLC32')
# 2) Product (ie, 'SLC' and 'SecureLinx Console Manager')
# 3) Number of ports (8, 16, 32, 48)
# 4) Company name (ie, 'Lantronix')
# 5) Timezone list on datetime.htm
#
# This script should be run in the directory containing the web pages, and it
# will not run unless one of these two files exist in that directory: 
#   .customize              - grabs all values from flash, and updates 
#                             config.xml and all web pages
#   .customize-override     - for custom OEM configurations.  behaves the
#                             same as .customize, except that the values for
#                             company, product and model are not grabbed from
#                             flash - the values in xml/config.xml are used.
#                             this enabled an OEM to put custom values in
#                             xml/config.xml and use those to override the
#                             values in flash.
#
# This script expects the file orig/htm.tar.gz to contain the original raw
# html files (48 port version, no customization or timezone list).
#
# Revision History
#
# 9/18/03 CAG Original version
#

override=0
rootdir=/
tmpdir=/tmp/xwstmp
wwwdir=${rootdir}www
bootdir=$wwwdir
bindir=${rootdir}bin
dualboot=0

if [ $# -eq 1 ]; then
    if [ $1 -eq 1 -o $1 -eq 2 ] ; then
        dualboot=$1
        bootdir=${rootdir}www/$dualboot
    fi
fi

# does .customize or .customize-override exist?
if [ ! -f $bootdir/.customize -a ! -f $bootdir/.customize-override ]; then
    echo "Exiting, neither $bootdir/.customize or $bootdir/.customize-override exist."
    exit 0
fi

# does .customize and .customize-override exist?  -- not clear what to do
if [ -f $bootdir/.customize -a -f $bootdir/.customize-override ]; then
    echo "Exiting, $bootdir/.customize and $bootdir/.customize-override both exist."
    exit 0
fi

if [ -f $bootdir/.customize-override ]; then
    override=1
fi

if [ ! -d $tmpdir ]; then
    /bin/mkdir $tmpdir
fi

cd $wwwdir

#
# restore the original .htm files 
#
echo -e "Customizing Web Pages..."
/bin/echo_xconsole "Customizing Web Pages..."
/bin/rm -f *.htm
/bin/tar xzf orig/htm.tar.gz 

#
# get product, model, number of ports from /proc
#
product=SLC
productlong="SecureLinx Console Manager"
numports=$(/bin/grep nports /etc/ltrx/info | /bin/awk -F= '{ print $2 }' )
modelnum=$(/bin/grep model /etc/ltrx/info | /bin/awk -F= '{ print $2 }' )
if [ $numports -eq 48 ] ; then
    model=SLC48
elif [ $numports -eq 32 ] ; then
    model=SLC32
elif [ $numports -eq 16 ] ; then
    if [ $modelnum -eq 15 -o $modelnum -eq 16 ] ; then
        model=SLB1684
        product=SLB
        productlong="SecureLinx Branch Office Manager"
    else
        model=SLC16
    fi
elif [ $numports -eq 8 ] ; then
    if [ $modelnum -eq 13 -o $modelnum -eq 14 ] ; then
        model=SLB884
        product=SLB
        productlong="SecureLinx Branch Office Manager"
    else
        model=SLC8
    fi
else
    echo "Invalid number of ports $numports."
    exit 0
fi


# copy original factory defaults file
/bin/rm -f $tmpdir/config.xml

# update values in config.xml
if [ $override -eq 0 ]; then
    # we are not going to override the flash values with what's in config.xml
    # so populate config.xml with flash values
    /bin/rm -f xml/config.xml 
    /bin/cp xml-defaults/config.xml xml
    /bin/sed -i "s/<MODEL>SLC48/<MODEL>$model/g
         s/<PRODUCT>SLC/<PRODUCT>$product/g
         s/<PRODUCTLONG>SecureLinx Console Manager/<PRODUCTLONG>$productlong/g" xml/config.xml
    /bin/rm -f xml-defaults/config.xml
    /bin/cp xml/config.xml xml-defaults
fi

# read values from config.xml, to be used for remainder of customizations
company=$(/bin/grep COMPANY xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
product=$(/bin/grep \<PRODUCT\> xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
productlong=$(/bin/grep PRODUCTLONG xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
model=$(/bin/grep MODEL xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
website=$(/bin/grep WEBSITE xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
tech_email=$(/bin/grep TECH_EMAIL xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
echo -e "Customizing with $company, $product, $productlong,\n                 $model, $website, $tech_email"
/bin/echo_xconsole "Customizing with $company, $product, $productlong, $model, $website, $tech_email"
##echo -n "Do you want to continue? [y] "
##read ans
##if [ $ans ]; then
##    if [ $ans != "y" ] ; then
##        echo "Aborting."
##        exit
##    fi
##fi

#
# Update the web pages
# By default, all web pages are setup for a SLC48
#

# update web pages with only title, product, and model name updates
for f in *.htm ; do
    /bin/sed "s/Lantronix SLC48/$company $model/g
         s/SecureLinx Console Manager/$productlong/g
         s/SLC48/$model/g
         s/www.lantronix.com/$website/g
         s/SLC /$product /g" $f > $tmpdir/$f
done


# copy updated web pages from temp directory to current directory
/bin/rm -f *.htm
/bin/cp $tmpdir/*.htm .


# for SLB, change the online help links
#if [ $product == "SLB" ] ; then
#    for f in *.htm ; do
#        /bin/sed -i "s/SLC_Online_Help_for_Web/SLB_Online_Help_for_Web/g
#                     s/SLC_Network.htm/SLB_Network.htm/g" $f
#    done
#fi


#
# Update pages that are dependent on number of device ports
#
# if [ $numports -ne 48 ]; then
    # remove extra ports from banner.htm
    /bin/awk "{ if ( NF >= 8 && \$6 == \"Device\" && \$7 == \"Port\" )
             { if ( \$8 <= $numports ) 
                 print \$0 }
           else 
             print \$0 }" banner.htm > $tmpdir/banner.htm
    /bin/rm -f banner.htm
    /bin/cp $tmpdir/banner.htm .
    # remove buttons from devports.htm
    /bin/awk "{ if ( NF >= 3 && \$3 == \"name=btn16\" )
             { if ( $numports >= 32 ) 
                 print \$0 }
           else if ( NF >= 3 && \$3 == \"name=btn32\" )
             { if ( $numports >= 32 ) 
                 print \$0 }
           else if ( NF >= 3 && \$3 == \"name=btn48\" )
             { if ( $numports >= 48 ) 
                 print \$0 }
           else if ( NF >= 2 && \$2 == \"SLC8\" )
             { if ( $numports == 8 )
                 print \$0 }
           else 
             print \$0 }" devports.htm > $tmpdir/devports.htm
    /bin/rm -f devports.htm
    /bin/cp $tmpdir/devports.htm .
# fi

#
# Do web telnet/ssh jnlp files
#
cd network
/bin/rm -f *.jnlp *.htm
/bin/tar xzf ../orig/network.tar.gz 
for f in *.jnlp ; do
    /bin/sed -i "s/Lantronix SLC48/$company $model/g" $f
done
for f in *.htm ; do
    /bin/sed -i "s/Lantronix SLC48/$company $model/g
                 s/SecureLinx Console Manager/$productlong/g
                 s/SLC48/$model/g
                 s/www.lantronix.com/$website/g
                 s/SLC /$product /g" $f 
done


#
# update timezones in datetime.htm and qsetup.htm
#

echo "Generating timezones..."
/bin/echo_xconsole "Generating timezones..."
# get list of timezone files
cd /usr/share/zoneinfo
/bin/rm -f $tmpdir/zonefiles $tmpdir/zonefiles2
/bin/find . -type f -print | egrep -v 'right|posix|\.tab' | sort > $tmpdir/zonefiles
# remove leading './'
/bin/sed 's/\.\///g' $tmpdir/zonefiles > $tmpdir/zonefiles2
cd $wwwdir
$bindir/xws_inserttz
retval=$?
if [ $retval -eq 0 ]; then
    /bin/rm -f datetime.htm
    /bin/cp $tmpdir/datetime.htm .
    /bin/rm -f qsetup.htm
    /bin/cp $tmpdir/qsetup.htm .
elif [ $retval -eq 1 ]; then
    echo "Cannot open original datetime.htm or qsetup.htm"
elif [ $retval -eq 2 ]; then
    echo "Cannot open new datetime.htm or qsetup.htm"
elif [ $retval -eq 3 ]; then
    echo "Cannot open temporary timezone list file"
fi

#
# trim devports.xml if number of ports is less than 48
#
if [ $numports -ne 48 ]; then
    $bindir/xws_trimdevports $numports
    retval=$?
    if [ $retval -eq 0 ]; then
        /bin/rm -f xml/devports.xml xml-defaults/devports.xml
        /bin/cp $tmpdir/devports.xml xml
        /bin/cp $tmpdir/devports.xml xml-defaults
    elif [ $retval -eq 1 ]; then
        echo "Wrong arguments passed to xws_trimdevports"
    elif [ $retval -eq 2 ]; then
        echo "Cannot open original xml/devports.xml"
    elif [ $retval -eq 3 ]; then
        echo "Cannot open new devports.xml"
    fi
fi

#
# Cleanup
#
/bin/rm -rf $tmpdir
/bin/rm -f $bootdir/.customize $bootdir/.customize-override 

if [ $dualboot -ne 0 ]; then
    # zip up customized pages into htm directory
    /bin/rm -f $bootdir/htm.tgz $bootdir/network.tgz
    tar czf $bootdir/htm.tgz *.htm
    cd network
    tar czf $bootdir/network.tgz *.jnlp *.htm
fi

#
# Make sure updates are written to disk
#
/bin/sync

