#! /bin/bash
# firmware version: 5.3b
#
# script to update any 5.X single or dual boot SLC firmware to 5.3b
#
# this script expects:
#     the firmware update file to be unpacked in /www
#     /tmp/boot to point to the current boot bank
#     /tmp/db-copyconfig to exist if the current config should be copied
#
# this script updates the other boot bank:
#     mounts the other R/O partition and updates it
#     updates both the shared (in /www) and non-shared (in /www/{1,2} files
#     re-customizes any web pages that have been updated
#     deletes any obsolete files
#     for any existing DTDs that have been updated, converts the associated
#         XML file to conform to the new DTD
#     updates the version in /www/xml/server.xml
#     sets the eeprom to boot from the updated bank
#     removes the patch file 
#     


#
# Main program
#

fwrev=5.3b
fwrevmajor=5
fwrevminor=3
scrdir=/www/slcupdate-db-5.3b
tmpdir=/tmp/xwstmp
htmdir=/tmp/xwshtm
tee="/bin/tee -a /tmp/pre-update.txt"

/bin/rm -f /tmp/pre-update.txt
/bin/touch /tmp/pre-update.txt

# don't do anything if we are on a single boot system
echo "Checking boot configuration..." | $tee
if [ ! -d /www/1 -o ! -d /www/2 ] ; then 
    prod=$(/bin/grep \<PRODUCT\> /www/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    echo "Single-boot $prod, firmware update requires reboot." | $tee
    exit 0
fi

prod=$(/bin/grep \<PRODUCT\> /tmp/boot/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')

if [ ! -d $scrdir ] ; then
    echo "Firmware update directory $scrdir does not exist." | $tee
    echo "Firmware update failed." | $tee
    exit 1
fi

# make sure everything is flushed before we get started
/bin/sync

cd $scrdir

echo "Installing update for $prod firmware revision $fwrev." | $tee

#
# determine bank & partitions
# Bank 1: hda1 (/), hda2 (/www), hda3 (kernel)
# Bank 2: hda5 (/), hda2 (/www), hda6 (kernel)

/bin/rm -f /tmp/nextbank

rootdev=$(/bin/grep dev /etc/fstab | grep -v nfs | grep "0 0" | /bin/awk ' { print $1 }')
bootlink=$(/bin/readlink /tmp/boot)
if [ $rootdev == "/dev/hda1" -a $bootlink == "/www/1" ] ; then
    currbank=1
    nextbank=2
elif [ $rootdev == "/dev/hda5" -a $bootlink == "/www/2" ] ; then
    currbank=2
    nextbank=1
else
    echo "Error: cannot determine current boot bank." | $tee
    # remove the patch file so XMS will not reboot for firmware update
    /bin/rm -f $scrdir.tgz
    exit 1
fi

ln -s /www/$nextbank /tmp/nextbank

echo "Checking firmware version..." | $tee
currrev=$(/bin/grep REV /tmp/nextbank/xml/server.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< '{ print $1 }')
if [ $currrev == $fwrev ] ; then
    echo "" | $tee
    echo "*** This $prod is already running firmware version $fwrev on Bank $nextbank." | $tee
    echo "*** The patch will not be installed." | $tee
    echo "" | $tee
    # remove the patch file so XMS will not reboot for firmware update
    /bin/rm -f $scrdir.tgz
    exit 0
fi

currmajor=$(/bin/grep REV /tmp/nextbank/xml/server.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< '{ print $1 }' | /bin/awk -F'[A-Za-z]' '{ print $1 }' | /bin/awk -F. '{ print $1 }' )
currminor=$(/bin/grep REV /tmp/nextbank/xml/server.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< '{ print $1 }' | /bin/awk -F'[A-Za-z]' '{ print $1 }' | /bin/awk -F. '{ print $2 }' )

if [ $currmajor -lt 5 ] ; then
    echo "" | $tee
    echo "*** This patch may only be installed on" | $tee
    echo "*** a $prod running firmware version 5.0 or later." | $tee
    echo "*** Please upgrade to one of these firmware versions," | $tee
    echo "*** then reapply this patch." | $tee
    echo "" | $tee
    # remove the patch file so XMS will not reboot for firmware update
    /bin/rm -f $scrdir.tgz
    exit 0
fi

installpatch=1
if [ $currmajor -gt $fwrevmajor ] ; then
    installpatch=0
elif [ $currmajor -eq $fwrevmajor -a $currminor -gt $fwrevminor ] ; then
    installpatch=0
fi
if [ $installpatch -eq 0 ] ; then
    echo "" | $tee
    echo "*** The currently installed firmware version" | $tee
    echo "*** is greater than the patch version." | $tee
    echo "*** The patch will not be installed." | $tee
    echo "" | $tee
    # remove the patch file so XMS will not reboot for firmware update
    /bin/rm -f $scrdir.tgz
    exit 0
fi

echo "Checking filesystem version..." | $tee
if [ -f /sbin/sfdisk ] ; then
    /sbin/sfdisk -l -uS /dev/hda > /tmp/sectors
else
    ./sbin/sfdisk -l -uS /dev/hda > /tmp/sectors
fi
start=$(/bin/grep hda3 /tmp/sectors | /bin/awk ' { print $2 }')
rm /tmp/sectors
if [ -f uImage.gz ] ; then
    if [ $start -eq 242688 ] ; then
        echo "" | $tee
        echo "*** This patch may not be installed on a $prod " | $tee
        echo "*** that originally ran firmware version 1.0." | $tee
        echo "*** Please contact Lantronix Tech Support." | $tee
        echo "" | $tee
        # remove the patch file so XMS will not reboot for firmware update
        /bin/rm -f $scrdir.tgz
        exit 0
    fi
fi

# make dirs if they do not exist; clear anything in them if they do exist
/bin/mkdir -p $tmpdir $htmdir
/bin/rm -f $tmpdir/* $htmdir/*

echo "Currently running Bank $currbank, applying firmware update to Bank $nextbank." | $tee

# if we are updating bank 1, copy the root partition from bank 2 to bank 1
if [ $currbank -eq 2 ] ; then
    echo "Copying root partition..." | $tee
    /bin/dd if=/dev/hda5 of=/dev/hda1 >> /tmp/pre-update.txt 2>&1
fi

if [ -d /tmp/mnt ] ; then
    if /bin/mount | grep -q /tmp/mnt ; then
        umount /tmp/mnt >> /tmp/pre-update.txt 2>&1
    fi
else
    mkdir -p /tmp/mnt
fi

if [ $currbank -eq 1 ] ; then
    mount -o rw /dev/hda5 /tmp/mnt >> /tmp/pre-update.txt 2>&1
else
    mount -o rw /dev/hda1 /tmp/mnt >> /tmp/pre-update.txt 2>&1
fi
retval=$?
if [ $retval -ne 0 ]; then
    echo "Error mounting Bank $nextbank R/O partition." | $tee
    echo "Patch installation halted." | $tee
    # remove the patch file so XMS will not reboot for firmware update
    /bin/rm -f $scrdir.tgz
    exit 0
fi

# copy correct fstab (cleanup from dd above)
if [ $currbank -eq 2 ] ; then
    /bin/cp fstab.hda1 /tmp/mnt/etc/fstab
fi

echo -n "Copying new and changed files..." | $tee

# tar files to the 1st partition
for d in bin boot etc lib root sbin usr var x15 ; do
    if [ -d $d ] ; then
        cd $d
        tar -cf - * | (cd /tmp/mnt/$d ; tar xf -)
        echo -n "." | $tee
        cd ..
    fi
done

# if there is a u-boot.bin file, touch .updateuboot so that the boot loader
# version will be checked at reboot
if [ -f bin/u-boot.bin ] ; then
    touch /tmp/nextbank/.updateuboot
fi

# for SLB, install the SLB SNMP MIB library
if [ -f lib/libnetsnmpmibs.so.5.1.1.slb -a $prod == "SLB" ] ; then
  /bin/rm -f /tmp/mnt/lib/libnetsnmpmibs.so.5.1.1
  /bin/mv /tmp/mnt/lib/libnetsnmpmibs.so.5.1.1.slb /tmp/mnt/lib/libnetsnmpmibs.so.5.1.1
fi

# lots of updates, let's flush again
/bin/sync

# tar files to the 2nd (read-write) partition
if [ -d www ] ; then
    cd www

    echo "." | $tee

    if [ -f network/jta20.jar ] ; then
        /bin/cp -a -f network/jta20.jar /tmp/nextbank/network
    fi
    if [ -f network/ssh2term.jar ] ; then
        /bin/cp -a -f network/ssh2term.jar /tmp/nextbank/network
    fi

    # customize web files
    # read values from config.xml, to be used for customizations
    # for dual-boot, use the values in the current boot bank, as they should
    # be the same as what is in the next boot bank (or will be in the next
    # boot bank, if it has never been booted)
    company=$(/bin/grep COMPANY /tmp/boot/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    product=$(/bin/grep \<PRODUCT\> /tmp/boot/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    productlong=$(/bin/grep PRODUCTLONG /tmp/boot/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    model=$(/bin/grep MODEL /tmp/boot/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    website=$(/bin/grep WEBSITE /tmp/boot/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    echo -e "Re-customizing with $company, $product, $productlong,\n                    $model, $website" | $tee

    # determine number of ports
    if [ $model == "SLC8" -o $model == "SLB884" ] ; then
        numports=8
    elif [ $model == "SLC16" -o $model == "SLB1684" ] ; then
        numports=16
    elif [ $model == "SLC32" ] ; then
        numports=32
    else
        numports=48
    fi

    # unpack current web pages for bank, customize updated pages
    # if this file does not exist, the updated pages will also be in
    # /tmp/nextbank/orig/htm.tar.gz
    if [ -f /tmp/nextbank/htm.tgz ] ; then
        tar xzf /tmp/nextbank/htm.tgz -C $htmdir

        # customize files to a temp dir, then copy to $htmdir
        for f in *.htm ; do
            /bin/sed "s/Lantronix SLC48/$company $model/g
                 s/SecureLinx Console Manager/$productlong/g
                 s/SLC48/$model/g
                 s/www.lantronix.com/$website/g
                 s/SLC /$product /g" $f > $tmpdir/$f
            /bin/rm -f $htmdir/$f
            /bin/cp $tmpdir/$f $htmdir
        done

        # Update pages that are dependent on number of device ports
        if [ -f banner.htm ] ; then
            # remove extra ports from banner.htm -
            # update the file that was just customized
            /bin/rm -f banner.htm $tmpdir/banner.htm
            /bin/cp $htmdir/banner.htm .
            /bin/awk "{ if ( NF >= 8 && \$6 == \"Device\" && \$7 == \"Port\" )
                     { if ( \$8 <= $numports )
                         print \$0 }
                   else
                     print \$0 }" banner.htm > $tmpdir/banner.htm
            /bin/rm -f $htmdir/banner.htm
            /bin/cp $tmpdir/banner.htm $htmdir
        fi

        if [ -f devports.htm ] ; then
            # remove buttons from devports.htm 
            # update the file that was just updated
            /bin/rm -f devports.htm $tmpdir/devports.htm
            /bin/cp $htmdir/devports.htm .
            /bin/awk "{ if ( NF >= 3 && \$3 == \"name=btn16\" )
                     { if ( $numports >= 32 )
                         print \$0 }
                   else if ( NF >= 3 && \$3 == \"name=btn32\" )
                     { if ( $numports >= 32 )
                         print \$0 }
                   else if ( NF >= 3 && \$3 == \"name=btn48\" )
                     { if ( $numports >= 48 )
                         print \$0 }
                   else if ( NF >= 2 && \$2 == \"SLC8\" )
                     { if ( $numports == 8 )
                         print \$0 }
                   else
                     print \$0 }" devports.htm > $tmpdir/devports.htm
            /bin/rm -f $htmdir/devports.htm
            /bin/cp $tmpdir/devports.htm $htmdir
        fi

        if [ -f datetime.htm -o -f qsetup.htm ] ; then
            # populate timezones in datetime.htm 
            # update the file that was just updated
            dodatetime=0
            doqsetup=0
            if [ -f datetime.htm ] ; then
                dodatetime=1
            fi
            if [ -f qsetup.htm ] ; then
                doqsetup=1
            fi
            /bin/rm -f datetime.htm $tmpdir/datetime.htm
            /bin/cp $htmdir/datetime.htm .
            /bin/rm -f qsetup.htm $tmpdir/qsetup.htm
            /bin/cp $htmdir/qsetup.htm .

            echo "Updating timezones..." | $tee
            # get list of timezone files
            cd /usr/share/zoneinfo
            /bin/rm -f $tmpdir/zonefiles $tmpdir/zonefiles2
            /bin/find . -type f -print | egrep -v 'right|posix|\.tab' | sort > $tmpdir/zonefiles
            # remove leading './'
            /bin/sed 's/\.\///g' $tmpdir/zonefiles > $tmpdir/zonefiles2
            cd $scrdir/www
            /bin/xws_inserttz
            retval=$?
            if [ $retval -eq 0 ]; then
                if [ $dodatetime -eq 1 ] ; then
                    /bin/rm -f $htmdir/datetime.htm
                    /bin/cp $tmpdir/datetime.htm $htmdir
                fi
                if [ $doqsetup -eq 1 ] ; then
                    /bin/rm -f $htmdir/qsetup.htm
                    /bin/cp $tmpdir/qsetup.htm $htmdir
                fi
            elif [ $retval -eq 1 ]; then
                echo "Cannot open original datetime.htm or qsetup.htm" | $tee
            elif [ $retval -eq 2 ]; then
                echo "Cannot open new datetime.htm or qsetup.htm" | $tee
            elif [ $retval -eq 3 ]; then
                echo "Cannot open temporary timezone list file" | $tee
            fi
        fi

        # remove obsolete files
        /bin/rm -f $htmdir/authremote.htm $htmdir/menu.htm $htmdir/netgateway.htm

        # zip up and replace htm.tgz
        /bin/rm -f /tmp/nextbank/htm.tgz
        cd $htmdir
        tar czf /tmp/nextbank/htm.tgz *.htm

        cd $scrdir/www
        /bin/rm -f $tmpdir/* $htmdir/*

    fi

    # web telnet pages
    if [ -d network -a -f /tmp/nextbank/network.tgz ] ; then
        # unpack current web pages for bank
        tar xzf /tmp/nextbank/network.tgz -C $htmdir
        # as of 5.2, the .htm web pages (expect for webtelnet.ssh and
        # webssh.htm) are no longer needed
        /bin/rm -f $htmdir/*.htm
        # customize .jnlp files
        cd network
        for f in *.jnlp *.htm ; do
            /bin/sed "s/Lantronix SLC48/$company $model/g" $f > $tmpdir/$f
            /bin/rm -f $htmdir/$f
            /bin/cp $tmpdir/$f $htmdir
        done
        /bin/rm -f /tmp/nextbank/network.tgz
        cd $htmdir
        tar czf /tmp/nextbank/network.tgz *.jnlp *.htm
        cd $scrdir/www
    fi

    # we are done with these files in the patch
    rm -rf *.htm network

    # for all xml files that are new since 5.0, but prior to 5.3, see if there
    # is already an existing version which needs to be saved and then converted
    # to 5.3 format
    #   xpb.xml         - added in 5.2
    #   hostlist.xml    - added in 5.3
    for x in xpb hostlist ; do
        if [ -f /tmp/nextbank/xml/$x.xml ] ; then
            /bin/cp /tmp/nextbank/xml/$x.xml /tmp
        fi
    done

    # copy all the new/changed files that are not shared between boot banks
    for d in js orig xml xml-defaults ; do
        if [ -d $d ] ; then
            tar -cf - $d | (cd /www/$nextbank ; tar xf -)
        fi
    done

    # restore any saved xml files
    for x in xpb hostlist ; do
        if [ -f /tmp/$x.xml ] ; then
            /bin/rm -f /tmp/nextbank/xml/$x.xml
            /bin/cp /tmp/$x.xml /tmp/nextbank/xml
        fi
    done

    if [ -d help ] ; then
        tar -cf - help | (cd /www/$nextbank ; tar xf -)
    fi
    if [ -d /www/$nextbank/help-slb ] ; then
        rm -rf /www/$nextbank/help-slb
    fi

    # trim xml-defaults devports.xml if number of ports is less than 48
    if [ -f xml-defaults/devports.xml -a $numports -ne 48 ]; then
        mkdir /tmp/xml
        cp xml-defaults/devports.xml /tmp/xml
        cd /tmp
        /bin/xws_trimdevports $numports
        retval=$?
        if [ $retval -eq 0 ]; then
            /bin/rm -f /tmp/nextbank/xml-defaults/devports.xml
            /bin/cp $tmpdir/devports.xml /tmp/nextbank/xml-defaults
        elif [ $retval -eq 1 ]; then
            echo "Wrong arguments passed to xws_trimdevports" | $tee
        elif [ $retval -eq 2 ]; then
            echo "Cannot open original xml/devports.xml" | $tee
        elif [ $retval -eq 3 ]; then
            echo "Cannot open new devports.xml" | $tee
        fi
        cd $scrdir/www
    fi

    rm -rf help help-slb js orig xml xml-defaults

    # copy all the new/changed files that are shared between boot banks
    # copying these *cannot* affect the current boot bank which is running
    tar -cf - * | (cd /www ; tar xf -)

    cd $scrdir

fi

echo -n "Deleting obsolete files..." | $tee

# delete files
# R/O partition: expected to be under /tmp/mnt
if [ -f delete_files_ro ] ; then
    for f in `cat delete_files_ro` ; do
        rm -f -r /tmp/mnt$f
        echo -n "." | $tee
    done
fi
# R/W partition: listed as /www/... (shared) or /tmp/nextbank/... (non-shared)
if [ -f delete_files_rw ] ; then
    for f in `cat delete_files_rw` ; do
        rm -f -r $f
        echo -n "." | $tee
    done
fi

echo "" | $tee

# another flush...
/bin/sync

# update kernel
if [ -f uImage.gz ] ; then
    echo "Updating kernel..." | $tee
    if [ $nextbank -eq 1 ] ; then
        /bin/gunzip -d < uImage.gz | /bin/dd of=/dev/hda3 >> /tmp/pre-update.txt 2>&1
    else
        /bin/gunzip -d < uImage.gz | /bin/dd of=/dev/hda6 >> /tmp/pre-update.txt 2>&1
    fi
fi

#
# convert all XML files to 5.3 format
#

if [ -f /tmp/db-copyconfig ] ; then
    # copy configuration from current bank before continuing
    mv -f /tmp/nextbank/xml/app.xml /tmp/nextbank/xml/app.xml-
    cp -f /www/$currbank/xml/*.xml /tmp/nextbank/xml
    mv -f /tmp/nextbank/xml/app.xml- /tmp/nextbank/xml/app.xml
    rm -rf /tmp/nextbank/xml/keys
    cp -r /www/$currbank/xml/keys /tmp/nextbank/xml/keys
    rm -f /tmp/db-copyconfig
    rm -f /tmp/nextbank/.configure /tmp/nextbank/.configureip
fi

cd $scrdir
for i in config devports kerberos ldap localusers network nis pccard radius remoteusers server services tacacs termport ; do
    case "$i" in
        config)         grepstr="SLB884"
                        xmlstr="Configuration"
                        ;;
        devports)       grepstr="SHOWLOG"
                        xmlstr="Device Ports"
                        ;;
        kerberos)       grepstr="ESCSEQ"
                        xmlstr="Kerberos"
                        ;;
        ldap)           grepstr="ESCSEQ"
                        xmlstr="LDAP"
                        ;;
        localusers)     grepstr="ALLOW_REUSE"
                        xmlstr="Local Users"
                        ;;
        network)        grepstr="ALTERNATE"
                        xmlstr="Network"
                        ;;
        nis)            grepstr="ESCSEQ"
                        xmlstr="NIS"
                        ;;
        pccard)         grepstr="DIALIN_HOSTLIST"
                        xmlstr="PC Card"
                        ;;
        radius)         grepstr="ESCSEQ"
                        xmlstr="RADIUS"
                        ;;
        remoteusers)    grepstr="ESCSEQ"
                        xmlstr="Remote Users"
                        ;;
        server)         grepstr="WEB_GADGET"
                        xmlstr="Maintenance"
                        ;;
        services)       grepstr="WEBLAUNCH"
                        xmlstr="Services"
                        ;;
        tacacs)         grepstr="ESCSEQ"
                        xmlstr="TACACS"
                        ;;
        termport)       grepstr="SHOWLOG"
                        xmlstr="Console Port"
                        ;;
        *)              ;;
    esac
    if /bin/grep -q $grepstr /tmp/nextbank/xml/$i.xml ; then
        echo "$xmlstr XML already converted, skipping..." | $tee
    else
        # run conversion program to convert XML to the new DTD
        echo "Converting $xmlstr XML..." | $tee
        ./convert_$i /tmp/nextbank/xml/$i.xml
        retval=$?
        if [ $retval -eq 0 ]; then
            /bin/rm -f /tmp/nextbank/xml/$i.xml
            /bin/cp /tmp/$i.xml /tmp/nextbank/xml
        elif [ $retval -eq 1 ]; then
            echo "Cannot open original /tmp/nextbank/xml/$i.xml" | $tee
        elif [ $retval -eq 2 ]; then
            echo "Cannot open new /tmp/$i.xml" | $tee
        fi
    fi
done


# update firmware revision in server.xml
/bin/sed -i "s/<REV>[\.a-zA-Z0-9]*<\/REV>/<REV>$fwrev<\/REV>/g" /tmp/nextbank/xml/server.xml >> /tmp/pre-update.txt 2>&1
/bin/sed -i "s/<REV>[\.a-zA-Z0-9]*<\/REV>/<REV>$fwrev<\/REV>/g" /tmp/nextbank/xml-defaults/server.xml >> /tmp/pre-update.txt 2>&1

# touch timestamp file
/bin/touch /tmp/nextbank/.timestamp

# flush updates
/bin/sync

# cleanup
umount /tmp/mnt >> /tmp/pre-update.txt 2>&1
rmdir /tmp/mnt
/bin/rm -rf $tmpdir $htmdir

# call eeprom to configure next boot bank and setup link to /www bank dir
/bin/eeprom -s $nextbank >> /dev/null 2>&1
rm -f /tmp/next /tmp/nextbank
ln -s /www/$nextbank /tmp/next

# remove the patch file so XMS will not reboot for firmware update
/bin/rm -f $scrdir.tgz

/bin/echo "Installation of $prod firmware revision $fwrev to Bank $nextbank complete." | $tee
/bin/echo "The $prod is configured to use Bank $nextbank at the next reboot." | $tee

# ADD "Update End: " MESSAGE?

exit 0
